@@ -15,9 +15,9 @@ maxlikefn<- function(z, x, INIT=FALSE, TERM=FALSE, data, effects=NULL,
         {
             effects <- getEffects(data)
         }
-        if (!is.data.frame(effects))
+		if(!inherits(effects, "sienaEffects"))
         {
-            stop('effects is not a data.frame')
+			stop("effects is not a legitimate Siena effects object")
         }
         effects <- effects[effects$include, ]
         z$theta <- effects$initialValue
