@@ -12,15 +12,29 @@
 #ifndef ALLEFFECTS_H_
 #define ALLEFFECTS_H_
 
+#include "AltersCovariateAvAltEffect.h"
 #include "AltersCovariateAverageEffect.h"
+#include "InAltersCovariateAverageEffect.h"
+#include "AltersCovariateAvSimEffect.h"
+#include "AltersCovariateTotSimEffect.h"
+#include "AltersDist2CovariateAverageEffect.h"
+#include "AltersInDist2CovariateAverageEffect.h"
+#include "AntiIsolateEffect.h"
 #include "AverageAlterEffect.h"
+#include "AverageAlterDist2Effect.h"
+#include "AverageAlterInDist2Effect.h"
+#include "AverageInAlterEffect.h"
 #include "AverageReciprocatedAlterEffect.h"
 #include "BalanceEffect.h"
 #include "BehaviorInteractionEffect.h"
 #include "BetweennessEffect.h"
+#include "BothDegreesEffect.h"
 #include "CovariateAlterEffect.h"
+#include "CovariateDiffEffect.h"
+#include "CovariateDiffEgoEffect.h"
 #include "CovariateEgoAlterEffect.h"
 #include "CovariateEgoEffect.h"
+#include "CovariateEgoSquaredEffect.h"
 #include "CovariateIndirectTiesEffect.h"
 #include "CovariateSimilarityEffect.h"
 #include "DenseTriadsBehaviorEffect.h"
@@ -28,6 +42,10 @@
 #include "DenseTriadsSimilarityEffect.h"
 #include "DensityEffect.h"
 #include "DistanceTwoEffect.h"
+#include "DoubleDegreeBehaviorEffect.h"
+#include "DoubleInPopEffect.h"
+#include "DoubleRecDegreeBehaviorEffect.h"
+#include "DyadicCovariateAvAltEffect.h"
 #include "DyadicCovariateMainEffect.h"
 #include "DyadicCovariateReciprocityEffect.h"
 #include "FourCyclesEffect.h"
@@ -35,6 +53,8 @@
 #include "IndegreeEffect.h"
 #include "IndegreeActivityEffect.h"
 #include "IndegreePopularityEffect.h"
+#include "RecipdegreeActivityEffect.h"
+#include "RecipdegreePopularityEffect.h"
 #include "InInDegreeAssortativityEffect.h"
 #include "InIsolateDegreeEffect.h"
 #include "InOutDegreeAssortativityEffect.h"
@@ -48,6 +68,7 @@
 #include "JumpCovariateTransitiveTripletsEffect.h"
 #include "LinearShapeEffect.h"
 #include "MainCovariateEffect.h"
+#include "MaxAlterEffect.h"
 #include "NetworkInteractionEffect.h"
 #include "OutdegreeEffect.h"
 #include "OutdegreeActivityEffect.h"
@@ -57,16 +78,26 @@
 #include "OutOutDegreeAssortativityEffect.h"
 #include "PopularityAlterEffect.h"
 #include "QuadraticShapeEffect.h"
+#include "RecipdegreeActivityEffect.h"
+#include "RecipdegreePopularityEffect.h"
 #include "ReciprocalDegreeBehaviorEffect.h"
 #include "ReciprocatedSimilarityEffect.h"
 #include "ReciprocityEffect.h"
 #include "SameCovariateEffect.h"
+#include "SameCovariateActivityEffect.h"
+#include "SameCovariateFourCyclesEffect.h"
+#include "HomCovariateActivityEffect.h"
+#include "AlterCovariateActivityEffect.h"
 #include "SameCovariateTransitiveTripletsEffect.h"
+#include "HomCovariateTransitiveTripletsEffect.h"
 #include "SimilarityEffect.h"
+#include "SimilarityWEffect.h"
 #include "SimilarityTransitiveTripletsEffect.h"
 #include "ThreeCyclesEffect.h"
+#include "ThresholdShapeEffect.h"
 #include "TransitiveMediatedTripletsEffect.h"
 #include "TransitiveReciprocatedTripletsEffect.h"
+#include "TransitiveReciprocatedTriplets2Effect.h"
 #include "TransitiveTiesEffect.h"
 #include "TransitiveTriadsEffect.h"
 #include "TransitiveTripletsEffect.h"
