@@ -18,6 +18,8 @@
 #include "model/EffectInfo.h"
 #include "model/variables/BehaviorVariable.h"
 
+using namespace std;
+
 namespace siena
 {
 
@@ -59,6 +61,11 @@ void BehaviorEffect::initialize(const Data * pData,
 	this->lvalues = pState->behaviorValues(name);
 }
 
+void BehaviorEffect::initialize(const Data *pData,
+	State *pState, State *pSimulatedState, int period, Cache *pCache)
+{
+	initialize(pData, pState, period, pCache);
+}
 
 /**
  * Returns the number of actors of the behavior variable associated with this
@@ -134,20 +141,40 @@ double BehaviorEffect::similarityMean() const
  */
 double BehaviorEffect::evaluationStatistic(double * currentValues)
 {
+	return this->evaluationStatistic(currentValues, false).first;
+}
+
+pair<double,double *> BehaviorEffect::evaluationStatistic(double * currentValues, bool needActorStatistics)
+{
+	this->initializeStatisticCalculation();
 	double statistic = 0;
 	int n = this->n();
 
+	double * actorStatistics = 0;
+	if(needActorStatistics)
+	{
+		 actorStatistics = new double[n];
+	}
+
 	for (int i = 0; i < n; i++)
 	{
 		this->preprocessEgo(i);
 		if (!this->missing(this->period(), i) &&
 			!this->missing(this->period() + 1, i))
 		{
-			statistic += this->egoStatistic(i, currentValues);
+			if(needActorStatistics)
+			{
+				actorStatistics[i] = this->egoStatistic(i, currentValues);
+				statistic += actorStatistics[i];
+			}
+			else
+			{
+				statistic += this->egoStatistic(i, currentValues);
+			}
 		}
 	}
-
-	return statistic;
+	this->cleanupStatisticCalculation();
+	return make_pair(statistic,actorStatistics);
 }
 
 
@@ -170,21 +197,41 @@ double BehaviorEffect::egoStatistic(int ego, double * currentValues)
 double BehaviorEffect::endowmentStatistic(const int * difference,
 	double * currentValues)
 {
+	return endowmentStatistic(difference, currentValues, false).first;
+}
+
+pair<double, double * > BehaviorEffect::endowmentStatistic(const int * difference, double * currentValues, bool needActorStatistics)
+{
+	this->initializeStatisticCalculation();
 	double statistic = 0;
 	int n = this->n();
 
+	double * actorStatistics = 0;
+	if(needActorStatistics)
+	{
+		actorStatistics = new double[n];
+	}
+
 	for (int i = 0; i < n; i++)
 	{
 		this->preprocessEgo(i);
 		if (!this->missing(this->period(), i))
 		{
-			statistic += this->egoEndowmentStatistic(i, difference,
-				currentValues);
+			if(needActorStatistics)
+			{
+				actorStatistics[i] = this->egoEndowmentStatistic(i, difference, currentValues);
+				statistic += actorStatistics[i];
+			}
+			else
+			{
+				statistic += this->egoEndowmentStatistic(i, difference, currentValues);
+			}
 		}
 	}
-
-	return statistic;
+	this->cleanupStatisticCalculation();
+	return make_pair(statistic, actorStatistics);
 }
+
 /**
  * Returns the statistic corresponding the given ego as part of
  * the endowment function with respect to an initial behavior
@@ -197,7 +244,6 @@ double BehaviorEffect::egoEndowmentStatistic(int i, const int * difference,
 		this->pEffectInfo()->effectName());
 }
 
-
 /**
  * Returns the statistic corresponding to this effect as part of
  * the creation function.
@@ -209,6 +255,12 @@ double BehaviorEffect::egoEndowmentStatistic(int i, const int * difference,
  */
 double BehaviorEffect::creationStatistic(int * difference,
 	double *currentValues)
+{
+	return creationStatistic(difference, currentValues, false).first;
+}
+
+pair<double, double * > BehaviorEffect::creationStatistic(int * difference,
+	double *currentValues, bool needActorStatistics)
 {
 	// Here we use a trick. The creation statistics are very similar to the
 	// endowmnent statistics, but instead of summing over all actors with
@@ -216,24 +268,39 @@ double BehaviorEffect::creationStatistic(int * difference,
 	// values. So we just reverse the differences and call the endowment
 	// statistic.
 
+	this->initializeStatisticCalculation();
 	int n = this->n();
+	double statistic = 0;
+	double * actorStatistics = 0;
+	for (int i = 0; i < n; i++)
+	{
+		difference[i]  = -difference[i];
+	}
+
+	if(needActorStatistics)
+	{
+		pair<double, double *> p = this->endowmentStatistic(difference, currentValues, needActorStatistics);
+		statistic = p.first;
+		actorStatistics = p.second;
+		for (int i = 0; i < n; i++)
+		{
+			actorStatistics[i]  = -actorStatistics[i];
+		}
+	}
+	else
+	{
+		statistic = this->endowmentStatistic(difference, currentValues);
+	}
 
 	for (int i = 0; i < n; i++)
 	{
 		difference[i]  = -difference[i];
 	}
 
-	double statistic = this->endowmentStatistic(difference, currentValues);
-
-	for (int i = 0; i < n; i++)
-	{
-		difference[i]  = -difference[i];
-	}
-
-	return -statistic;
+	this->cleanupStatisticCalculation();
+	return make_pair(-statistic, actorStatistics);
 }
 
-
 /**
  * Does the necessary preprocessing work for calculating the probabilities
  * for a specific ego. This method must be invoked before
@@ -243,4 +310,21 @@ void BehaviorEffect::preprocessEgo(int ego)
 {
 	this->lego = ego;
 }
+
+/**
+ * This method is called at the start of the calculation of the 
+ * evaluationStatistic, endowmentStatistic, and creationStatistic
+ */
+void BehaviorEffect::initializeStatisticCalculation()
+{
+}
+
+/**
+ * This method is called at the end of the calculation of the 
+ * evaluationStatistic, endowmentStatistic, and creationStatistic.
+ */
+void BehaviorEffect::cleanupStatisticCalculation()
+{
+}
+
 }
