@@ -20,6 +20,8 @@
 #include "model/EpochSimulation.h"
 #include "model/variables/BehaviorVariable.h"
 
+using namespace std;
+
 namespace siena
 {
 
@@ -27,19 +29,37 @@ namespace siena
  * Constructor.
  */
 CovariateDependentNetworkEffect::CovariateDependentNetworkEffect(
-	const EffectInfo * pEffectInfo) : NetworkEffect(pEffectInfo)
-{
-	this->lpConstantCovariate = 0;
-	this->lpChangingCovariate = 0;
-	this->lpBehaviorData = 0;
-	this->lvalues = 0;
+		const EffectInfo * pEffectInfo) :
+		NetworkEffect(pEffectInfo), //
+		lSimulatedOffset(0), //
+		lpConstantCovariate(0), //
+		lpChangingCovariate(0), //
+		lpBehaviorData(0), //
+		lvalues(0) {
 }
 
+/**
+ * Constructor.
+ *
+ * @param pEffectInfo The effect info.
+ * @param simulatedState If `true` the value(), missing() and similarity()
+ *        functions uses the simulated state, if any or the value at the end
+ *        of the period.
+ */
+CovariateDependentNetworkEffect::CovariateDependentNetworkEffect(
+		const EffectInfo * pEffectInfo, bool simulatedState) :
+		NetworkEffect(pEffectInfo), //
+		lSimulatedOffset(simulatedState ? 1 : 0), //
+		lpConstantCovariate(0), //
+		lpChangingCovariate(0), //
+		lpBehaviorData(0), //
+		lvalues(0) {
+}
 
 /**
  * Initializes this effect.
  * @param[in] pData the observed data
- * @param[in] pState the current state of the dependent variables
+ * @param[in] pState the state of the dependent variables at the beginning of the period
  * @param[in] period the period of interest
  * @param[in] pCache the cache object to be used to speed up calculations
  */
@@ -61,33 +81,55 @@ void CovariateDependentNetworkEffect::initialize(const Data * pData,
 		!(this->lpBehaviorData && this->lvalues))
 	{
 		throw logic_error("Covariate or dependent behavior variable '" +
-			name +
-			"' expected.");
+			name + "' expected.");
 	}
 }
 
 
+/**
+ * Initializes this effect.
+ * @param[in] pData the observed data
+ * @param[in] pState the state of the dependent variables at the beginning of the period
+ * @param[in] pSimulatedState the current simulated state of the dependent variables
+ * @param[in] period the period of interest
+ * @param[in] pCache the cache object to be used to speed up calculations
+ */
+void CovariateDependentNetworkEffect::initialize(const Data* pData,
+		State* pState, State* pSimulatedState, int period, Cache* pCache) {
+	NetworkEffect::initialize(pData, pState, pSimulatedState, period, pCache);
+	const string name = this->pEffectInfo()->interactionName1();
+
+	// For covariates lSimulatedOffset is applied in values().
+	this->lpConstantCovariate = pData->pConstantCovariate(name);
+	this->lpChangingCovariate = pData->pChangingCovariate(name);
+
+	this->lpBehaviorData = pData->pBehaviorData(name);
+	// If the covariate is a behaviour take the right state.
+	if (lSimulatedOffset == 1) {
+		this->lvalues = pSimulatedState->behaviorValues(name);
+	} else {
+		this->lvalues = pState->behaviorValues(name);
+	}
+
+	if (!this->lpConstantCovariate && !this->lpChangingCovariate
+			&& !(this->lpBehaviorData && this->lvalues)) {
+		throw logic_error(
+				"Covariate or dependent behavior variable '" + name + "' expected.");
+	}
+}
+
 /**
  * Returns the covariate value for the given actor.
  */
-double CovariateDependentNetworkEffect::value(int i) const
+double CovariateDependentNetworkEffect::value(const int i) const
 {
-	double value = 0;
-
-	if (this->lpConstantCovariate)
-	{
-		value = this->lpConstantCovariate->value(i);
+	if (this->lpConstantCovariate) {
+		return this->lpConstantCovariate->value(i);
 	}
-	else if (this->lpChangingCovariate)
-	{
-		value = this->lpChangingCovariate->value(i, this->period());
+	if (this->lpChangingCovariate) {
+		return this->lpChangingCovariate->value(i, this->period() + lSimulatedOffset);
 	}
-	else
-	{
-		value = this->lvalues[i] - this->lpBehaviorData->overallMean();
-	}
-
-	return value;
+  return this->lvalues[i] - this->lpBehaviorData->overallMean();
 }
 
 
@@ -96,22 +138,13 @@ double CovariateDependentNetworkEffect::value(int i) const
  */
 bool CovariateDependentNetworkEffect::missing(int i) const
 {
-	bool missing = false;
-
-	if (this->lpConstantCovariate)
-	{
-		missing = this->lpConstantCovariate->missing(i);
+	if (this->lpConstantCovariate) {
+		return this->lpConstantCovariate->missing(i);
 	}
-	else if (this->lpChangingCovariate)
-	{
-		missing = this->lpChangingCovariate->missing(i, this->period());
+	if (this->lpChangingCovariate) {
+		return this->lpChangingCovariate->missing(i, this->period() + lSimulatedOffset);
 	}
-	else
-	{
-		missing = this->lpBehaviorData->missing(this->period(), i);
-	}
-
-	return missing;
+  return this->lpBehaviorData->missing(this->period() + lSimulatedOffset, i);
 }
 
 
