@@ -11,6 +11,11 @@
 #ifndef EFFECT_H_
 #define EFFECT_H_
 
+// TODO Header files should not do 'using namespace'.  This might lead to
+// namespace conflicts which are very hard to resolve.  Removeing this breaks
+// almost all effects since the <effect>.cpp files depend on it.
+using namespace std;
+
 namespace siena
 {
 
@@ -42,6 +47,7 @@ public:
 	virtual ~Effect();
 
 	inline double parameter() const;
+	void parameter(double value) ;
 
 	inline int period() const;
 	inline Cache * pCache() const;
