@@ -13,6 +13,7 @@
 #define NETWORKEFFECT_H_
 
 #include "Effect.h"
+#include <utility>
 
 namespace siena
 {
@@ -39,6 +40,7 @@ class NetworkEffect : public Effect
 {
 	friend class NetworkInteractionEffect;
 	friend class EffectFactory;
+	friend class BothDegreesEffect;
 
 public:
 	NetworkEffect(const EffectInfo * pEffectInfo);
@@ -47,6 +49,8 @@ public:
 		State * pState,
 		int period,
 		Cache * pCache);
+	virtual void initialize(const Data * pData, State * pState,
+			State * pSimulatedState, int period, Cache * pCache);
 
 	inline const Network * pNetwork() const;
 	inline const NetworkLongitudinalData * pData() const;
@@ -63,13 +67,17 @@ public:
 	virtual double calculateContribution(int alter) const = 0;
 
 	virtual double evaluationStatistic();
+	virtual pair <double, double * > evaluationStatistic(bool needActorStatistics);
 	virtual double endowmentStatistic(Network * pLostTieNetwork);
+	virtual pair <double, double * > endowmentStatistic(Network * pLostTieNetwork, bool needActorStatistics);
 	virtual double creationStatistic(Network * pGainedTieNetwork);
+	virtual pair <double, double * > creationStatistic(Network * pGainedTieNetwork, bool needActorStatistics);
 
 	virtual bool egoEffect() const;
 
 protected:
 	virtual double statistic(const Network * pSummationTieNetwork);
+	virtual pair <double, double * > statistic(const Network * pSummationTieNetwork, bool needActorStatistics);
 	virtual void initializeStatisticCalculation();
 	virtual void onNextEgo(int ego);
 	virtual double egoStatistic(int ego,
