@@ -31,12 +31,16 @@ StructuralRateEffect::StructuralRateEffect(const NetworkVariable * pVariable,
 	this->lpVariable = pVariable;
 	this->ltype = type;
 
-	double possibleDegree = max(this->lpVariable->n(),
+	double possibleDegree = std::max(this->lpVariable->n(),
 		this->lpVariable->m());
 	if (this->ltype == INVERSE_OUT_DEGREE_RATE)
 	{
 		this->lpTable = new EffectValueTable(possibleDegree, invertor);
 	}
+	else if (this->ltype == LOG_OUT_DEGREE_RATE)
+	{
+		this->lpTable = new EffectValueTable(possibleDegree, logarithmer);
+	}
 	else
 	{
 		this->lpTable = new EffectValueTable(possibleDegree, identity);
@@ -45,7 +49,6 @@ StructuralRateEffect::StructuralRateEffect(const NetworkVariable * pVariable,
 	this->lpTable->parameter(parameter);
 }
 
-
 /**
  * Destructor.
  */
@@ -55,7 +58,6 @@ StructuralRateEffect::~StructuralRateEffect()
 	this->lpTable = 0;
 }
 
-
 /**
  * Returns the contribution of this effect for the given actor.
  */
@@ -67,6 +69,7 @@ double StructuralRateEffect::value(int i)
 	{
 		case OUT_DEGREE_RATE:
 		case INVERSE_OUT_DEGREE_RATE:
+		case LOG_OUT_DEGREE_RATE:
 			return this->lpTable->value(pNetwork->outDegree(i));
 
 		case IN_DEGREE_RATE:
@@ -77,7 +80,7 @@ double StructuralRateEffect::value(int i)
 				(((OneModeNetwork *) pNetwork)->reciprocalDegree(i)));
 	}
 
-	throw new logic_error("Unexpected structural rate effect type");
+	throw std::logic_error("Unexpected structural rate effect type");
 }
 /**
  * Stores the parameter for the structural rate effect.
@@ -95,5 +98,4 @@ double StructuralRateEffect::parameter() const
 	return this->lpTable->parameter();
 }
 
-
 }
