@@ -27,7 +27,8 @@ enum StructuralRateEffectType
 	OUT_DEGREE_RATE,
 	IN_DEGREE_RATE,
 	RECIPROCAL_DEGREE_RATE,
-	INVERSE_OUT_DEGREE_RATE
+	INVERSE_OUT_DEGREE_RATE,
+	LOG_OUT_DEGREE_RATE
 };
 
 
