@@ -51,6 +51,8 @@ public:
 	inline int secondOutTieValue(int alter) const;
 
 	inline MixedEgocentricConfigurationTable * pTwoPathTable() const;
+	inline MixedEgocentricConfigurationTable * pInStarTable() const;
+	inline MixedEgocentricConfigurationTable * pOutStarTable() const;
 
 private:
 	// The first network this cache object is associated with
@@ -72,6 +74,12 @@ private:
 
 	// The number of two-paths from the ego to each of the alters
 	MixedEgocentricConfigurationTable * lpTwoPathTable;
+
+	// The number of in-two-stars from ego and alter to the middle actor.
+	MixedEgocentricConfigurationTable * lpInStarTable;
+
+	// The number of out-two-stars from the middle actor to ego and alter.
+	MixedEgocentricConfigurationTable * lpOutStarTable;
 };
 
 
@@ -95,13 +103,22 @@ const Network * TwoNetworkCache::pSecondNetwork() const
 	return this->lpSecondNetwork;
 }
 
+/**
+ * Indicates if there is a tie from the ego to the given alter in the first
+ * network. This method runs in constant time.
+ */
+bool TwoNetworkCache::firstOutTieExists(int alter) const
+{
+	return (this->lfirstOutTieValues[alter] >= 1);
+}
+
 /**
  * Indicates if there is a tie from the ego to the given alter in the second
  * network. This method runs in constant time.
  */
 bool TwoNetworkCache::secondOutTieExists(int alter) const
 {
-	return this->lsecondOutTieValues[alter];
+	return (this->lsecondOutTieValues[alter] >= 1);
 }
 
 
@@ -133,6 +150,24 @@ MixedEgocentricConfigurationTable * TwoNetworkCache::pTwoPathTable() const
 	return this->lpTwoPathTable;
 }
 
+/**
+ * Returns the table storing the number of in-two-stars from ego
+ * and alter to the middle actor.
+ */
+MixedEgocentricConfigurationTable * TwoNetworkCache::pInStarTable() const
+{
+	return this->lpInStarTable;
+}
+
+/**
+ * Returns the table storing the number of out-two-stars to ego
+ * and alter from the middle actor.
+ */
+MixedEgocentricConfigurationTable * TwoNetworkCache::pOutStarTable() const
+{
+	return this->lpOutStarTable;
+}
+
 }
 
 #endif /* TWONETWORKCACHE_H_ */
