@@ -37,6 +37,8 @@
 #include "model/variables/EffectValueTable.h"
 #include "network/IncidentTieIterator.h"
 
+using namespace std;
+
 namespace siena
 {
 
@@ -81,6 +83,7 @@ DependentVariable::DependentVariable(string name,
 	 	this->lsettingProbs = 0;
 	}
 	this->lstepType = -1;
+	this->lpChangeContribution = 0;
 }
 
 
@@ -127,64 +130,60 @@ void DependentVariable::initializeRateFunction()
 
 			//if (parameter != 0)
 			//{
-				ConstantCovariate * pConstantCovariate =
-					pData->pConstantCovariate(interactionName);
-				ChangingCovariate * pChangingCovariate =
-					pData->pChangingCovariate(interactionName);
-				const BehaviorVariable * pBehaviorVariable =
-					(const BehaviorVariable *)
-					this->lpSimulation->pVariable(interactionName);
+			ConstantCovariate * pConstantCovariate =
+				pData->pConstantCovariate(interactionName);
+			ChangingCovariate * pChangingCovariate =
+				pData->pChangingCovariate(interactionName);
+			const BehaviorVariable * pBehaviorVariable =
+				(const BehaviorVariable *)
+				this->lpSimulation->pVariable(interactionName);
 
-				if (pConstantCovariate)
+			if (pConstantCovariate)
+			{
+				if (this->lpActorSet->pOriginalActorSet() !=
+					pConstantCovariate->pActorSet())
 				{
-					if (this->lpActorSet->pOriginalActorSet() !=
-						pConstantCovariate->pActorSet())
-					{
-						throw domain_error("Mismatch of actor sets");
-					}
-
-					this->lconstantCovariateParameters[pConstantCovariate] =
-						parameter;
-					this->lconstantCovariateScores[pConstantCovariate] = 0;
-					this->lconstantCovariateSumTerm[pConstantCovariate] = 0;
-					this->lconstantCovariateModelBSumTerm[pConstantCovariate]
-						= 0;
+					throw domain_error("Mismatch of actor sets");
 				}
-				else if (pChangingCovariate)
+
+				this->lconstantCovariateParameters[pConstantCovariate] =
+					parameter;
+				this->lconstantCovariateScores[pConstantCovariate] = 0;
+				this->lconstantCovariateSumTerm[pConstantCovariate] = 0;
+				this->lconstantCovariateModelBSumTerm[pConstantCovariate]
+					= 0;
+			}
+			else if (pChangingCovariate)
+			{
+				if (this->lpActorSet->pOriginalActorSet() !=
+					pChangingCovariate->pActorSet())
 				{
-					if (this->lpActorSet->pOriginalActorSet() !=
-						pChangingCovariate->pActorSet())
-					{
-						throw domain_error("Mismatch of actor sets");
-					}
-
-					this->lchangingCovariateParameters[pChangingCovariate] =
-						parameter;
-					this->lchangingCovariateScores[pChangingCovariate] = 0;
-					this->lchangingCovariateSumTerm[pChangingCovariate] = 0;
-					this->lchangingCovariateModelBSumTerm[pChangingCovariate]
-						= 0;
+					throw domain_error("Mismatch of actor sets");
 				}
-				else if (pBehaviorVariable)
+
+				this->lchangingCovariateParameters[pChangingCovariate] =
+					parameter;
+				this->lchangingCovariateScores[pChangingCovariate] = 0;
+				this->lchangingCovariateSumTerm[pChangingCovariate] = 0;
+				this->lchangingCovariateModelBSumTerm[pChangingCovariate]
+					= 0;
+			}
+			else if (pBehaviorVariable)
+			{
+				if (this->lpActorSet != pBehaviorVariable->pActorSet())
 				{
-					if (this->lpActorSet != pBehaviorVariable->pActorSet())
-					{
-						throw domain_error("Mismatch of actor sets");
-					}
-
-					this->lbehaviorVariableParameters[pBehaviorVariable] =
-						parameter;
-					this->lbehaviorVariableScores[pBehaviorVariable] = 0;
-					this->lbehaviorVariableSumTerm[pBehaviorVariable] = 0;
-					this->lbehaviorVariableModelBSumTerm[pBehaviorVariable]
-						= 0;
+					throw domain_error("Mismatch of actor sets");
 				}
-				else
-					throw logic_error(
-						"No individual covariate named '" +
-						interactionName +
-						"'.");
-				//}
+
+				this->lbehaviorVariableParameters[pBehaviorVariable] =
+					parameter;
+				this->lbehaviorVariableScores[pBehaviorVariable] = 0;
+				this->lbehaviorVariableSumTerm[pBehaviorVariable] = 0;
+				this->lbehaviorVariableModelBSumTerm[pBehaviorVariable] = 0;
+			} else
+				throw logic_error(
+					"No individual covariate named '" + interactionName + "'.");
+			//}
 		}
 		else if (rateType == "structural")
 		{
@@ -194,22 +193,22 @@ void DependentVariable::initializeRateFunction()
 
 			if (interactionName == "")
 			{
-				 pVariable = dynamic_cast<const NetworkVariable *>(this);
+				pVariable = dynamic_cast<const NetworkVariable *>(this);
 			}
 			else
 			{
-				 pVariable = dynamic_cast<const NetworkVariable *>(
-					this->lpSimulation->pVariable(interactionName));
+				pVariable = dynamic_cast<const NetworkVariable *>(
+						this->lpSimulation->pVariable(interactionName));
 			}
 
 			if (!pVariable)
 			{
 				throw logic_error("The structural rate effect " +
-					effectName +
-					" for dependent variable " +
-					this->name() +
-					" refers to a non-existing network variable " +
-					interactionName);
+						effectName +
+						" for dependent variable " +
+						this->name() +
+						" refers to a non-existing network variable " +
+						interactionName);
 			}
 
 			if (effectName == "outRate")
@@ -220,9 +219,8 @@ void DependentVariable::initializeRateFunction()
 				}
 
 				this->lstructuralRateEffects.push_back(
-					new StructuralRateEffect(pVariable,
-						OUT_DEGREE_RATE,
-						parameter));
+						new StructuralRateEffect(pVariable, OUT_DEGREE_RATE,
+							parameter));
 				this->loutDegreeScores[pVariable] = 0;
 				this->loutDegreeSumTerm[pVariable] = 0;
 				this->loutDegreeModelBSumTerm[pVariable] = 0;
@@ -235,9 +233,8 @@ void DependentVariable::initializeRateFunction()
 				}
 
 				this->lstructuralRateEffects.push_back(
-					new StructuralRateEffect(pVariable,
-						IN_DEGREE_RATE,
-						parameter));
+						new StructuralRateEffect(pVariable, IN_DEGREE_RATE,
+							parameter));
 				this->linDegreeScores[pVariable] = 0;
 				this->linDegreeSumTerm[pVariable] = 0;
 			}
@@ -246,7 +243,7 @@ void DependentVariable::initializeRateFunction()
 				if (!pVariable->oneModeNetwork())
 				{
 					throw std::invalid_argument(
-						"One-mode network variable expected");
+							"One-mode network variable expected");
 				}
 
 				if (this->lpActorSet != pVariable->pSenders())
@@ -255,9 +252,8 @@ void DependentVariable::initializeRateFunction()
 				}
 
 				this->lstructuralRateEffects.push_back(
-					new StructuralRateEffect(pVariable,
-						RECIPROCAL_DEGREE_RATE,
-						parameter));
+						new StructuralRateEffect(pVariable,
+							RECIPROCAL_DEGREE_RATE, parameter));
 				this->lreciprocalDegreeScores[pVariable] = 0;
 				this->lreciprocalDegreeSumTerm[pVariable] = 0;
 			}
@@ -269,13 +265,26 @@ void DependentVariable::initializeRateFunction()
 				}
 
 				this->lstructuralRateEffects.push_back(
-					new StructuralRateEffect(pVariable,
-						INVERSE_OUT_DEGREE_RATE,
-						parameter));
+						new StructuralRateEffect(pVariable,
+							INVERSE_OUT_DEGREE_RATE, parameter));
 				this->linverseOutDegreeScores[pVariable] = 0;
 				this->linverseOutDegreeSumTerm[pVariable] = 0;
 				this->linverseOutDegreeModelBSumTerm[pVariable] = 0;
 			}
+			else if (effectName == "outRateLog")
+			{
+				if (this->lpActorSet != pVariable->pSenders())
+				{
+					throw std::invalid_argument("Mismatch of actor sets");
+				}
+
+				this->lstructuralRateEffects.push_back(
+						new StructuralRateEffect(pVariable, LOG_OUT_DEGREE_RATE,
+							parameter));
+				this->llogOutDegreeScores[pVariable] = 0;
+				this->llogOutDegreeSumTerm[pVariable] = 0;
+				this->llogOutDegreeModelBSumTerm[pVariable] = 0;
+			}
 			else
 			{
 				throw domain_error("Unexpected rate effect " + effectName);
@@ -549,7 +558,7 @@ void DependentVariable::initialize(int period)
 				this->lsettingRates[i] =
 					this->pSimulation()->pModel()->
 					settingRateParameter(pNetworkData,
-						rSettingNames[i], period);
+							rSettingNames[i], period);
 				sum += this->lsettingRates[i];
 			}
 			for (unsigned i = 0;
@@ -618,7 +627,7 @@ void DependentVariable::calculateRates()
 		{
 		    this->calculateScoreSumTerms();
 		}
-		if(this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+		if(this->symmetric() && this->networkModelTypeB())
 		{
 		    this->ltotalRate = this->totalRate() * this->totalRate() -
 				sumRatesSquared;
@@ -878,7 +887,7 @@ void DependentVariable::accumulateRateScores(double tau,
 	{
 		this->lbasicRateScore += 1.0 / this->basicRate();
 		if (this->symmetric() &&
-			this->pSimulation()->pModel()->modelTypeB())
+			this->networkModelTypeB())
 		{
 			throw logic_error("model type b");
 			//this->lbasicRateScore += 1.0 / this->basicRate();
@@ -886,27 +895,31 @@ void DependentVariable::accumulateRateScores(double tau,
 	}
 	this->lbasicRateScore -= this->totalRate() * tau / this->basicRate();
 
-	if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+	if (this->symmetric() && this->networkModelTypeB())
 	{
 		throw logic_error("model type b");
 		this->lbasicRateScore -= this->totalRate() * tau / this->basicRate();
 	}
 
-	// Update scores for setting rates
-	NetworkLongitudinalData * pNetworkData =
-		dynamic_cast<NetworkLongitudinalData *>(this->pData());
-	const std::vector< string > & rSettingNames  =
-		pNetworkData->rSettingNames();
-
-	for (int i = 0; i < this->lnumberSettings; i++)
+	// TODO Current settings implementation is only for networks.
+	if (this->networkVariable())
 	{
-		if (this == pSelectedVariable && this->lstepType == i)
+		// Update scores for setting rates
+		NetworkLongitudinalData * pNetworkData =
+			dynamic_cast<NetworkLongitudinalData *>(this->pData());
+		const std::vector< string > & rSettingNames  =
+			pNetworkData->rSettingNames();
+
+		for (int i = 0; i < this->lnumberSettings; i++)
 		{
-			this->lsettingRateScores[rSettingNames[i]] +=
-				1.0 / this->lsettingRates[i];
+			if (this == pSelectedVariable && this->lstepType == i)
+			{
+				this->lsettingRateScores[rSettingNames[i]] += 1.0
+					/ this->lsettingRates[i];
+			}
+			this->lsettingRateScores[rSettingNames[i]] -= this->totalRate() * tau /
+				this->lsettingRates[i];
 		}
-		this->lsettingRateScores[rSettingNames[i]] -= this->totalRate() * tau /
-			this->lsettingRates[i];
 	}
 
 	// Update scores for covariate dependent rate parameters
@@ -1018,6 +1031,22 @@ void DependentVariable::accumulateRateScores(double tau,
 		iter->second -= this->linverseOutDegreeSumTerm[iter->first] * tau;
 	}
 
+	for (std::map<const NetworkVariable *, double>::iterator iter =
+			this->llogOutDegreeScores.begin();
+		iter != this->llogOutDegreeScores.end();
+		iter++)
+	{
+		const Network * pNetwork = iter->first->pNetwork();
+
+		if (this == pSelectedVariable)
+		{
+			iter->second += logarithmer(pNetwork->outDegree(selectedActor));
+		}
+
+		iter->second -= this->llogOutDegreeSumTerm[iter->first] * tau;
+	}
+
+
 	// Update scores for diffusion rate parameters
 
 	const vector<EffectInfo *> & rRateEffects =
@@ -1122,117 +1151,133 @@ void DependentVariable::accumulateRateScores(double tau,
 	const DependentVariable * pSelectedVariable,
 	int selectedActor, int alter)
 {
-	 switch (this->pSimulation()->pModel()->modelType())
-	 {
-	 case NOTUSED:
-	 case NORMAL:
-	 case AFORCE:
-	 case AAGREE:
-		 break;
-	 case BFORCE:
-	 case BAGREE:
-	 case BJOINT:
+	switch (this->networkModelType())
+	{
+		case NOTUSED:
+		case NORMAL:
+		case AFORCE:
+		case AAGREE:
+			break;
+		case BFORCE:
+		case BAGREE:
+		case BJOINT:
 
-		 // Update the score for the basic rate parameter
+			// Update the score for the basic rate parameter
 
-		 if (this == pSelectedVariable && this->successfulChange())
-		 {
-			 this->lbasicRateScore += 2.0 / this->basicRate();
-		 }
-		 this->lbasicRateScore -=
-			 2 * this->totalRate() * tau / this->basicRate();
+			if (this == pSelectedVariable && this->successfulChange())
+			{
+				this->lbasicRateScore += 2.0 / this->basicRate();
+			}
+			this->lbasicRateScore -=
+				2 * this->totalRate() * tau / this->basicRate();
 
-		 // Update scores for covariate dependent rate parameters
+			// Update scores for covariate dependent rate parameters
 
-		 for (std::map<const ConstantCovariate *,
-				  double>::iterator iter =
-				  this->lconstantCovariateScores.begin();
-			  iter != this->lconstantCovariateScores.end();
-			  iter++)
-		 {
-			 const ConstantCovariate * pCovariate = iter->first;
+			for (std::map<const ConstantCovariate *,
+					double>::iterator iter =
+					this->lconstantCovariateScores.begin();
+					iter != this->lconstantCovariateScores.end();
+					iter++)
+			{
+				const ConstantCovariate * pCovariate = iter->first;
 
-			 if (this == pSelectedVariable && this->successfulChange())
-			 {
-				 iter->second += pCovariate->value(selectedActor) +
-					 pCovariate->value(alter);
-			 }
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second += pCovariate->value(selectedActor) +
+						pCovariate->value(alter);
+				}
 
-			 iter->second -= tau *
-				 this->lconstantCovariateModelBSumTerm[pCovariate];
-		 }
-		 for (std::map<const ChangingCovariate *, double>::iterator iter =
-				  this->lchangingCovariateScores.begin();
-			  iter != this->lchangingCovariateScores.end();
-			  iter++)
-		 {
-			 const ChangingCovariate * pCovariate = iter->first;
+				iter->second -= tau *
+					this->lconstantCovariateModelBSumTerm[pCovariate];
+			}
+			for (std::map<const ChangingCovariate *, double>::iterator iter =
+					this->lchangingCovariateScores.begin();
+					iter != this->lchangingCovariateScores.end();
+					iter++)
+			{
+				const ChangingCovariate * pCovariate = iter->first;
 
-			 if (this == pSelectedVariable && this->successfulChange())
-			 {
-				 iter->second +=
-						 pCovariate->value(selectedActor, this->period()) +
-					 pCovariate->value(alter, this->period());
-			 }
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second +=
+						pCovariate->value(selectedActor, this->period()) +
+						pCovariate->value(alter, this->period());
+				}
 
-			 iter->second -= tau *
-				 this->lchangingCovariateModelBSumTerm[pCovariate];
-		 }
-		 for (std::map<const BehaviorVariable *,
-				  double>::iterator iter =
-				  this->lbehaviorVariableScores.begin();
-			  iter != this->lbehaviorVariableScores.end();
-			  iter++)
-		 {
-			 const BehaviorVariable * pBehavior = iter->first;
+				iter->second -= tau *
+					this->lchangingCovariateModelBSumTerm[pCovariate];
+			}
+			for (std::map<const BehaviorVariable *,
+					double>::iterator iter =
+					this->lbehaviorVariableScores.begin();
+					iter != this->lbehaviorVariableScores.end();
+					iter++)
+			{
+				const BehaviorVariable * pBehavior = iter->first;
 
-			 if (this == pSelectedVariable && this->successfulChange())
-			 {
-				 iter->second += pBehavior->value(selectedActor) +
-					 pBehavior->value(alter);
-			 }
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second += pBehavior->value(selectedActor) +
+						pBehavior->value(alter);
+				}
 
-			 iter->second -= tau *
-				 this->lbehaviorVariableModelBSumTerm[pBehavior];
-		 }
+				iter->second -= tau *
+					this->lbehaviorVariableModelBSumTerm[pBehavior];
+			}
 
-		 // Update scores for structural rate parameters:
-		 // only out and inverse out for model type b
+			// Update scores for structural rate parameters:
+			// only out and inverse out for model type b
 
-		 for (std::map<const NetworkVariable *, double>::iterator iter =
-				  this->loutDegreeScores.begin();
-			  iter != this->loutDegreeScores.end();
-			  iter++)
-		 {
-			 const Network * pNetwork = iter->first->pNetwork();
+			for (std::map<const NetworkVariable *, double>::iterator iter =
+					this->loutDegreeScores.begin();
+					iter != this->loutDegreeScores.end();
+					iter++)
+			{
+				const Network * pNetwork = iter->first->pNetwork();
 
-			 if (this == pSelectedVariable && this->successfulChange())
-			 {
-				 iter->second += pNetwork->outDegree(selectedActor) +
-					 pNetwork->outDegree(alter);
-			 }
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second += pNetwork->outDegree(selectedActor) +
+						pNetwork->outDegree(alter);
+				}
 
-			 iter->second -= tau * this->loutDegreeModelBSumTerm[iter->first];
-		 }
+				iter->second -= tau * this->loutDegreeModelBSumTerm[iter->first];
+			}
 
+			for (std::map<const NetworkVariable *, double>::iterator iter =
+					this->linverseOutDegreeScores.begin();
+					iter != this->linverseOutDegreeScores.end();
+					iter++)
+			{
+				const Network * pNetwork = iter->first->pNetwork();
 
-		 for (std::map<const NetworkVariable *, double>::iterator iter =
-				  this->linverseOutDegreeScores.begin();
-			  iter != this->linverseOutDegreeScores.end();
-			  iter++)
-		 {
-			 const Network * pNetwork = iter->first->pNetwork();
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second += invertor(pNetwork->outDegree(selectedActor))
+						+ invertor(pNetwork->outDegree(alter));
+				}
+				iter->second -= tau *
+					this->linverseOutDegreeModelBSumTerm[iter->first];
+			}
 
-			 if (this == pSelectedVariable && this->successfulChange())
-			 {
-				 iter->second += invertor(pNetwork->outDegree(selectedActor))
-					 + invertor(pNetwork->outDegree(alter));
-			 }
-			 iter->second -= tau *
-				 this->linverseOutDegreeModelBSumTerm[iter->first];
-		 }
-	 }
+			for (std::map<const NetworkVariable *, double>::iterator iter =
+					this->llogOutDegreeScores.begin();
+					iter != this->llogOutDegreeScores.end();
+					iter++)
+			{
+				const Network * pNetwork = iter->first->pNetwork();
+
+				if (this == pSelectedVariable && this->successfulChange())
+				{
+					iter->second += logarithmer(pNetwork->outDegree(selectedActor))
+						+ logarithmer(pNetwork->outDegree(alter));
+				}
+				iter->second -= tau *
+					this->llogOutDegreeModelBSumTerm[iter->first];
+			}
+	}
 }
+
 /**
  * Calculates the rate score sum terms for this variable.
  *
@@ -1254,7 +1299,7 @@ void DependentVariable::calculateScoreSumTerms()
 		for (int i = 0; i < this->n(); i++)
 		{
 			timesRate += pCovariate->value(i) * this->lrate[i];
-			if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+			if (this->symmetric() && this->networkModelTypeB())
 			{
 				timesRateSquared += pCovariate->value(i) * this->lrate[i] *
 					this->lrate[i];
@@ -1276,7 +1321,7 @@ void DependentVariable::calculateScoreSumTerms()
 		for (int i = 0; i < this->n(); i++)
 		{
 			timesRate += pCovariate->value(i, this->period()) * this->lrate[i];
-			if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+			if (this->symmetric() && this->networkModelTypeB())
 			{
 				timesRateSquared += pCovariate->value(i, this->period()) *
 					this->lrate[i] * this->lrate[i];
@@ -1299,7 +1344,7 @@ void DependentVariable::calculateScoreSumTerms()
 		for (int i = 0; i < this->n(); i++)
 		{
 			timesRate += pBehavior->value(i) * this->lrate[i];
-			if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+			if (this->symmetric() && this->networkModelTypeB())
 			{
 				timesRateSquared += pBehavior->value(i) * this->lrate[i] *
 					this->lrate[i];
@@ -1339,7 +1384,7 @@ void DependentVariable::calculateScoreSumTerms()
 		for (int i = 0; i < this->n(); i++)
 		{
 			timesRate += pNetwork->outDegree(i) * this->lrate[i];
-			if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+			if (this->symmetric() && this->networkModelTypeB())
 			{
 				timesRateSquared += pNetwork->outDegree(i) * this->lrate[i] *
 					this->lrate[i];
@@ -1378,7 +1423,7 @@ void DependentVariable::calculateScoreSumTerms()
 		for (int i = 0; i < this->n(); i++)
 		{
 			timesRate += invertor(pNetwork->outDegree(i)) * this->lrate[i];
-			if (this->symmetric() && this->pSimulation()->pModel()->modelTypeB())
+			if (this->symmetric() && this->networkModelTypeB())
 			{
 				timesRateSquared += invertor(pNetwork->outDegree(i)) *
 					this->lrate[i] * this->lrate[i];
@@ -1390,6 +1435,30 @@ void DependentVariable::calculateScoreSumTerms()
 
 	}
 
+	for (std::map<const NetworkVariable *, double>::iterator iter =
+			 this->llogOutDegreeScores.begin();
+		 iter != this->llogOutDegreeScores.end();
+		 iter++)
+	{
+		const Network * pNetwork = iter->first->pNetwork();
+
+		double timesRate = 0;
+		double timesRateSquared = 0;
+		for (int i = 0; i < this->n(); i++)
+		{
+			timesRate += logarithmer(pNetwork->outDegree(i)) * this->lrate[i];
+			if (this->symmetric() && this->networkModelTypeB())
+			{
+				timesRateSquared += logarithmer(pNetwork->outDegree(i)) *
+					this->lrate[i] * this->lrate[i];
+			}
+		}
+		this->llogOutDegreeSumTerm[iter->first] = timesRate;
+		this->llogOutDegreeModelBSumTerm[iter->first] = 2 *
+			(this->ltotalRate * timesRate - timesRateSquared);
+
+	}
+
 	// Update scores for diffusion rate parameters.
 
 	const vector<EffectInfo *> & rRateEffects =
@@ -1627,6 +1696,23 @@ double DependentVariable::inverseOutDegreeScore(
 	return iter->second;
 }
 
+double DependentVariable::logOutDegreeScore(
+	const NetworkVariable * pNetworkData) const
+{
+	map<const NetworkVariable *, double>::const_iterator iter =
+		this->llogOutDegreeScores.find(pNetworkData);
+
+	if (iter == this->llogOutDegreeScores.end())
+	{
+		throw invalid_argument(
+			string("Unknown network: ") +
+			"The given log outdegree rate effect is not " +
+			"part of the model.");
+	}
+
+	return iter->second;
+}
+
 /**
  * Calculates the value of the diffusion rate effect for the given actor.
  */
@@ -1936,6 +2022,34 @@ bool DependentVariable::symmetric() const
 	return false;
 }
 
+/**
+ * Returns  the network model type.
+ */
+NetworkModelType DependentVariable::networkModelType() const
+{
+	// This method is overridden in NetworkVariable. Here we return NORMAL.
+	return NORMAL;
+}
+
+
+/**
+ * Returns  the behavioral model type.
+ */
+BehaviorModelType DependentVariable::behaviorModelType() const
+{
+	// This method is overridden in BehaviorVariable. Here we return RESTRICT.
+	return RESTRICT;
+}
+
+/**
+ * Returns whether the model type is one of the symmetric type b models.
+ */
+bool DependentVariable::networkModelTypeB() const
+{
+	// This method is overridden in NetworkVariable. Here we return false.
+	return false;
+}
+
 /**
  * Returns if there are any constraints on the permitted changes of this
  * variable.
