@@ -15,11 +15,11 @@
 /**
  *  Does one forward simulation for all the data by period within group
  */
-SEXP model(SEXP DERIV, SEXP DATAPTR, SEXP SEEDS,
+SEXP forwardModel(SEXP DERIV, SEXP DATAPTR, SEXP SEEDS,
 	SEXP FROMFINITEDIFF, SEXP MODELPTR, SEXP EFFECTSLIST,
 	SEXP THETA, SEXP RANDOMSEED2, SEXP RETURNDEPS, SEXP NEEDSEEDS,
-	SEXP USESTREAMS, SEXP ADDCHAINTOSTORE, SEXP NEEDCHANGECONTRIBUTIONS);
-
+	SEXP USESTREAMS, SEXP ADDCHAINTOSTORE, SEXP RETURNCHAINS, SEXP RETURNLOGLIK,
+	SEXP RETURNACTORSTATISTICS, SEXP RETURNCHANGECONTRIBUTIONS);
 /** Does some MH steps for a specified group and period.
  *  For multiple periods, the loop is always constructed in R.
  */
