@@ -15,17 +15,20 @@
  */
 
 #include <vector>
+#include <cstring>
 #include <R_ext/Random.h>
 #include <Rinternals.h>
 #include "siena07internals.h"
 #include "siena07utilities.h"
 #include "data/Data.h"
 #include "data/NetworkLongitudinalData.h"
+#include "data/BehaviorLongitudinalData.h"
 #include "model/Model.h"
 #include "model/ml/Chain.h"
 #include "model/ml/MiniStep.h"
 #include "model/State.h"
 #include "model/StatisticCalculator.h"
+#include "model/EffectInfo.h"
 #include "data/ActorSet.h"
 #include "model/ml/MLSimulation.h"
 #include "model/variables/DependentVariable.h"
@@ -396,7 +399,7 @@ SEXP effects(SEXP RpData, SEXP EFFECTSLIST)
 
 SEXP interactionEffects(SEXP RpModel, SEXP EFFECTSLIST)
 {
-   	Model * pModel = (Model *) R_ExternalPtrAddr(RpModel);
+	Model * pModel = (Model *) R_ExternalPtrAddr(RpModel);
 
 	// get the column names from the names attribute
 
@@ -495,11 +498,12 @@ SEXP deleteModel(SEXP RpModel)
 }
 
 /**
- *  sets up the model options of MAXDEGREE, CONDITIONAL
+ *  sets up the model options of MAXDEGREE, UNIVERSALOFFSET, CONDITIONAL
  */
 SEXP setupModelOptions(SEXP DATAPTR, SEXP MODELPTR, SEXP MAXDEGREE,
+	SEXP UNIVERSALOFFSET,
 	SEXP CONDVAR, SEXP CONDTARGETS, SEXP PROFILEDATA, SEXP PARALLELRUN,
-	SEXP MODELTYPE, SEXP SIMPLERATES)
+	SEXP MODELTYPE, SEXP BEHMODELTYPE, SEXP SIMPLERATES, SEXP NORMSETRATES)
 {
 	/* get hold of the data vector */
 	vector<Data *> * pGroupData = (vector<Data *> *)
@@ -509,6 +513,10 @@ SEXP setupModelOptions(SEXP DATAPTR, SEXP MODELPTR, SEXP MAXDEGREE,
 	/* get hold of the model object */
 	Model * pModel = (Model *) R_ExternalPtrAddr(MODELPTR);
 
+	if(!isNull(NORMSETRATES)){
+		pModel->normalizeSettingRates(*(LOGICAL(NORMSETRATES)));
+	}
+
 	int totObservations = totalPeriods(*pGroupData);
 
 	pModel->numberOfPeriods(totObservations);
@@ -550,15 +558,63 @@ SEXP setupModelOptions(SEXP DATAPTR, SEXP MODELPTR, SEXP MAXDEGREE,
 			}
 		}
 	}
+	/* get names vector for UniversalOffset */
+	if (!isNull(UNIVERSALOFFSET))
+	{
+		SEXP Names = getAttrib(UNIVERSALOFFSET, R_NamesSymbol);
+
+		for (int group = 0; group < nGroups; group++)
+		{
+			for (int i = 0; i < length(Names); i++)
+			{
+				Data * pData = (*pGroupData)[group];
+				NetworkLongitudinalData * pNetworkData =
+					pData->pNetworkData(CHAR(STRING_ELT(Names, i)));
+				pNetworkData->universalOffset(REAL(UNIVERSALOFFSET)[i]);
+			}
+		}
+	}
 	/* set the parallel run flag on the model */
 	if (!isNull(PARALLELRUN))
 	{
 		pModel->parallelRun(true);
 	}
+	/* get names vector for modeltype */
 	if (!isNull(MODELTYPE))
 	{
-		pModel->modelType(asInteger(MODELTYPE));
+		SEXP Names = getAttrib(MODELTYPE, R_NamesSymbol);
+
+		for (int group = 0; group < nGroups; group++)
+		{
+			for (int i = 0; i < length(Names); i++)
+			{
+				Data * pData = (*pGroupData)[group];
+				NetworkLongitudinalData * pNetworkData =
+					pData->pNetworkData(CHAR(STRING_ELT(Names, i)));
+				pNetworkData->modelType(INTEGER(MODELTYPE)[i]);
+			}
+		}
 	}
+	/* get names vector for modeltype */
+	if (!isNull(BEHMODELTYPE))
+	{
+		SEXP Names = getAttrib(BEHMODELTYPE, R_NamesSymbol);
+
+		for (int group = 0; group < nGroups; group++)
+		{
+			for (int i = 0; i < length(Names); i++)
+			{
+				Data * pData = (*pGroupData)[group];
+				BehaviorLongitudinalData * pBehaviorData =
+					pData->pBehaviorData(CHAR(STRING_ELT(Names, i)));
+				pBehaviorData->behModelType(INTEGER(BEHMODELTYPE)[i]);
+			}
+		}
+	}
+//	if (!isNull(MODELTYPE))
+//	{
+//		pModel->modelType(asInteger(MODELTYPE));
+//	}
 	// print out Data for profiling
 	if (asInteger(PROFILEDATA))
 	{
@@ -575,7 +631,8 @@ SEXP setupModelOptions(SEXP DATAPTR, SEXP MODELPTR, SEXP MAXDEGREE,
  *  Gets target values relative to the input data
  */
 SEXP getTargets(SEXP DATAPTR, SEXP MODELPTR, SEXP EFFECTSLIST,
-	SEXP PARALLELRUN)
+	SEXP PARALLELRUN, SEXP RETURNACTORSTATISTICS,
+	SEXP RETURNSTATICCHANGECONTRIBUTIONS)
 {
 	/* get hold of the data vector */
 	vector<Data *> * pGroupData = (vector<Data *> *)
@@ -586,9 +643,14 @@ SEXP getTargets(SEXP DATAPTR, SEXP MODELPTR, SEXP EFFECTSLIST,
 
 	if (!isNull(PARALLELRUN))
 	{
+		//TODO is this correct?
 		pModel->parallelRun(true);
+		// Or shouldn't it be:
+		// 	if(asInteger(PARALLELRUN)==1)
+		//  {
+		//		pModel->parallelRun(true);
+		//  }
 	}
-
 	int nGroups = pGroupData->size();
 
 	int totObservations = totalPeriods(*pGroupData);
@@ -612,6 +674,54 @@ SEXP getTargets(SEXP DATAPTR, SEXP MODELPTR, SEXP EFFECTSLIST,
 	{
 		rfra[i] = 0;
 	}
+
+	int returnActorStatistics = 0;
+	if (!isNull(RETURNACTORSTATISTICS))
+	{
+		returnActorStatistics = asInteger(RETURNACTORSTATISTICS);
+	}
+	/* changeStats will contain the target statistics of individual actors */
+	SEXP actorStats;
+	PROTECT(actorStats =  allocVector(VECSXP,nGroups));
+
+	int returnStaticChangeContributions = 0;
+	if (!isNull(RETURNSTATICCHANGECONTRIBUTIONS))
+	{
+		returnStaticChangeContributions = asInteger(RETURNSTATICCHANGECONTRIBUTIONS);
+	}
+	/* changeStats will contain the tie flip contributions or the behavior change contributions of each effect on the decisions of all actors */
+	SEXP changeStats;
+	PROTECT(changeStats =  allocVector(VECSXP,nGroups));
+
+	SEXP NETWORKTYPES = 0;
+	if(returnStaticChangeContributions)
+	{
+		 NETWORKTYPES =  createRObjectAttributes(EFFECTSLIST, changeStats);
+		 int objEffects = length(NETWORKTYPES);
+
+		 for (int group = 0; group < nGroups; group++)
+		 {
+			SET_VECTOR_ELT(changeStats, group, allocVector(VECSXP, (*pGroupData)[group]->observationCount()));
+			for (int p = 0; p < (*pGroupData)[group]->observationCount(); p++)
+			{
+				SET_VECTOR_ELT(VECTOR_ELT(changeStats,group), p, allocVector(VECSXP,objEffects));
+			}
+		}
+	}
+	if(returnActorStatistics)
+	{
+		NETWORKTYPES =  createRObjectAttributes(EFFECTSLIST, actorStats);
+		int objEffects = length(NETWORKTYPES);
+
+		for (int group = 0; group < nGroups; group++)
+		{
+			SET_VECTOR_ELT(actorStats, group, allocVector(VECSXP, (*pGroupData)[group]->observationCount()));
+			for (int p = 0; p < (*pGroupData)[group]->observationCount(); p++)
+			{
+				SET_VECTOR_ELT(VECTOR_ELT(actorStats,group), p, allocVector(VECSXP,objEffects));
+			}
+		}
+	}
 	/* find the targets: for each data object separately:
 	   add them up on return to R (easier to check!) */
 	int periodFromStart = 0;
@@ -647,10 +757,114 @@ SEXP getTargets(SEXP DATAPTR, SEXP MODELPTR, SEXP EFFECTSLIST,
 			{
 				rfra[iii + effectNo] = statistic[effectNo];
 			}
+			if(returnActorStatistics)
+			{
+				StatisticCalculator Calculator (pData, pModel, &State, period, returnActorStatistics);
+				int actors = pData->rDependentVariableData()[0]->n();
+				vector<double *> actorStatistics;
+				getActorStatistics(EFFECTSLIST, &Calculator, &actorStatistics);
+				for(unsigned e = 0; e < actorStatistics.size(); e++)
+				{
+					SEXP actorStatsValues;
+					PROTECT(actorStatsValues = allocVector(REALSXP,actors));
+					double * astats = REAL(actorStatsValues);
+					for(int i = 0; i < actors; i++)
+					{
+						astats[i]= actorStatistics.at(e)[i];
+					}
+					SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(actorStats,group), period+1),e,actorStatsValues);
+					UNPROTECT(1);
+				}
+			}
+			if(returnStaticChangeContributions)
+			{
+				StatisticCalculator Calculator (pData, pModel, &State, period, returnActorStatistics, returnStaticChangeContributions);
+				int actors = pData->rDependentVariableData()[0]->n();
+				vector<vector<double * > > changeContributions;
+				getChangeContributionStatistics(EFFECTSLIST, &Calculator, &changeContributions);
+				for(unsigned e = 0; e < changeContributions.size(); e++)
+				{
+					SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(changeStats,group), period+1),e,allocVector(VECSXP,actors));
+					int choices;
+					if (strcmp(CHAR(STRING_ELT(NETWORKTYPES,e)), "behavior") == 0)
+					{
+						choices = 3;
+					}
+					else
+					{
+						choices = actors;
+					}
+					for(int actor = 0; actor < actors; actor++)
+					{
+						SEXP actorsVal;
+						PROTECT(actorsVal = allocVector(REALSXP,choices));
+						double * d = REAL(actorsVal);
+						for(int i = 0; i< length(actorsVal); i++)
+						{
+							d[i]=changeContributions.at(e).at(actor)[i];
+						}
+						SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(changeStats,group), period+1),e),actor,actorsVal);
+						UNPROTECT(1);
+					}
+				}
+			}
+		}
+		// get change contributions of the first observation
+		if(returnStaticChangeContributions)
+		{
+			State State (pData, 0);
+			vector<vector<double * > > changeContributions;
+			StatisticCalculator Calculator (pData, pModel, &State, 0, returnActorStatistics, returnStaticChangeContributions);
+			getChangeContributionStatistics(EFFECTSLIST, &Calculator, &changeContributions);
+			for(unsigned e = 0; e < changeContributions.size(); e++)
+			{
+				int actors = pData->rDependentVariableData()[0]->n();
+				SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(changeStats,group), 0),e,allocVector(VECSXP,actors));
+				int choices;
+				if (strcmp(CHAR(STRING_ELT(NETWORKTYPES,e)), "behavior") == 0)
+				{
+					choices = 3;
+				}
+				else
+				{
+					choices = actors;
+				}
+				for(int actor = 0; actor < actors; actor++)
+				{
+					SEXP actorsVal;
+					PROTECT(actorsVal = allocVector(REALSXP,choices));
+					double * d = REAL(actorsVal);
+					for(int i = 0; i< length(actorsVal); i++)
+					{
+						d[i]=changeContributions.at(e).at(actor)[i];
+					}
+					SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(changeStats,group), 0),e),actor,actorsVal);
+					UNPROTECT(1);
+				}
+			}
 		}
 	}
-	UNPROTECT(1);
-	return fra;
+	if(returnStaticChangeContributions)
+	{
+		UNPROTECT(3);
+		return changeStats;
+	}
+	else if(returnActorStatistics)
+	{
+//		SEXP ans;
+//		PROTECT(ans = allocVector(VECSXP, 2));
+//		SET_VECTOR_ELT(ans, 0, fra);
+//		SET_VECTOR_ELT(ans, 1, actorStats);
+//		UNPROTECT(4);
+//		return ans;
+		UNPROTECT(3);
+		return actorStats;
+	}
+	else
+	{
+		UNPROTECT(3);
+		return fra;
+	}
 }
 
 /**
@@ -658,9 +872,13 @@ SEXP getTargets(SEXP DATAPTR, SEXP MODELPTR, SEXP EFFECTSLIST,
  * Processes a complete set of data objects, creating a chain for each
  * period and returning the address.
  */
+/**
+  * NOTE; FOR SOME CONFIGURATIONS OF STRUCTURAL ZEROS
+ *  THIS RUNS INTO A HANG
+ */
 SEXP mlMakeChains(SEXP DATAPTR, SEXP MODELPTR,
-	SEXP PROBS, SEXP PRMIN, SEXP PRMIB, SEXP MINIMUMPERM,
-	SEXP MAXIMUMPERM, SEXP INITIALPERM)
+		SEXP PROBS, SEXP PRMIN, SEXP PRMIB, SEXP MINIMUMPERM,
+		SEXP MAXIMUMPERM, SEXP INITIALPERM, SEXP LOCALML)
 {
 	/* get hold of the data vector */
 	vector<Data *> * pGroupData = (vector<Data *> *)
@@ -707,6 +925,14 @@ SEXP mlMakeChains(SEXP DATAPTR, SEXP MODELPTR,
 	PROTECT(aborts = allocVector(VECSXP, totObservations));
 	GetRNGstate();
 
+	/* localML */
+	int localML = 0;
+	if (!isNull(LOCALML))
+	{
+		localML = asInteger(LOCALML);
+	}
+	pModel->localML(localML);
+
 	int periodFromStart = 0;
 
 	for (int group = 0; group < nGroups; group++)
@@ -765,9 +991,9 @@ SEXP mlMakeChains(SEXP DATAPTR, SEXP MODELPTR,
 			pModel->chainStore(*pChain, periodFromStart);
 
 			/* return chain as a list */
- 			SEXP ch1;
- 			PROTECT(ch1 = getChainList(*pChain));
-  			//PROTECT(ch1 = getChainDFPlus(*pChain, true));
+			SEXP ch1;
+			PROTECT(ch1 = getChainList(*pChain));
+			//PROTECT(ch1 = getChainDFPlus(*pChain, true));
 			SET_VECTOR_ELT(currentChains, periodFromStart, ch1);
 
 			/* get hold of the statistics for accept and reject */
@@ -823,7 +1049,7 @@ SEXP mlMakeChains(SEXP DATAPTR, SEXP MODELPTR,
  */
 SEXP mlInitializeSubProcesses(SEXP DATAPTR, SEXP MODELPTR,
 	SEXP PROBS, SEXP PRMIN, SEXP PRMIB, SEXP MINIMUMPERM,
-	SEXP MAXIMUMPERM, SEXP INITIALPERM, SEXP CHAINS)
+	SEXP MAXIMUMPERM, SEXP INITIALPERM, SEXP CHAINS, SEXP LOCALML)
 {
 	/* get hold of the data vector */
 	vector<Data *> * pGroupData = (vector<Data *> *)
@@ -860,6 +1086,14 @@ SEXP mlInitializeSubProcesses(SEXP DATAPTR, SEXP MODELPTR,
 
 	int periodFromStart = 0;
 
+	/* localML */
+	int localML = 0;
+	if (!isNull(LOCALML))
+	{
+		localML = asInteger(LOCALML);
+	}
+	pModel->localML(localML);
+
 	for (int group = 0; group < nGroups; group++)
 	{
 		Data * pData = (*pGroupData)[group];
