@@ -9,6 +9,7 @@
  *****************************************************************************/
 
 #include "Utils.h"
+#include <Rmath.h>
 #include <sstream>
 
 namespace siena
@@ -42,6 +43,14 @@ double invertor(int x)
 	return 1.0 / (x + 1);
 }
 
+/**
+ * Returns the logarithm of (<i>x</i> + 1).
+ */
+double logarithmer(int x)
+{
+	return log(x + 1);
+}
+
 
 /**
  * Creates an exception signaling about the use of an invalid iterator.
